@@ -10,7 +10,14 @@ use Carp;
 
 Mail::DKIM::TextWrap - text wrapping module written for use with DKIM
 
-=head1 SYNOPSIS
+=head1 SYNOPSIS (FOR MAIL::DKIM USERS)
+
+  use Mail::DKIM::TextWrap;
+
+Just add the above line to any program that uses L<Mail::DKIM::Signer>
+and your signatures will automatically be wrapped to 72 characters.
+
+=head1 SYNOPSIS (FOR OTHER USERS)
 
   my $output = "";
   my $tw = Mail::DKIM::TextWrap->new(
@@ -125,6 +132,7 @@ sub new
 		Swallow => qr/\s/,
 		Separator => "\n",
 		cur => 0,
+		may_break => 0,
 		soft_space => "",
 		word => "",
 		%args,
@@ -137,6 +145,8 @@ sub new
 #
 # cur - the last known column position
 #
+# may_break - nonzero if the current location allows a linebreak
+#
 # soft_space - contains added text that will not be printed if a linebreak
 #              occurs
 #
@@ -200,6 +210,11 @@ sub add
 		{
 			$word = $1 . $2;
 		}
+		elsif ($self->{NoBuffering})
+		{
+			$word = $self->{word};
+			$self->{word} = "";
+		}
 		else
 		{
 			last;
@@ -208,7 +223,7 @@ sub add
 		die "assertion failed" unless length($word) >= 1;
 
 		my $next_soft_space;
-		if ($word =~ s/($swallow)$//s)
+		if (defined($swallow) && $word =~ s/($swallow)$//s)
 		{
 			$next_soft_space = $1;
 		}
@@ -220,7 +235,7 @@ sub add
 		my $to_print = $self->{soft_space} . $word;
 		my $new_pos = _calculate_new_column($self->{cur}, $to_print);
 
-		if ($new_pos > $self->{Margin})
+		if ($new_pos > $self->{Margin} && $self->{may_break})
 		{
 			# what would happen if we put the separator in?
 			my $w_sep = _calculate_new_column($self->{cur},
@@ -240,6 +255,7 @@ sub add
 		$self->output($to_print);
 		$self->{soft_space} = $next_soft_space;
 		$self->{cur} = $new_pos;
+		$self->{may_break} = 1;
 	}
 }
 
@@ -273,9 +289,10 @@ the current break pattern.
 sub flush
 {
 	my $self = shift;
-	$self->output($self->{soft_space} . $self->{word});
-	$self->{soft_space} = "";
-	$self->{word} = "";
+
+	local $self->{NoBuffering} = 1;
+	local $self->{Swallow} = undef;
+	$self->add("");
 }
 
 sub output
